#include <stdio.h>
#include <stdlib.h>
#include <time.h>

void *memcpy_noopt(void *__restrict dst, const void *__restrict src, size_t count);
void *memcpy_mmx(void *__restrict dst, const void *__restrict src, size_t count);
void *memcpy_sse(void *__restrict dst, const void *__restrict src, size_t count);
void *memcpy_sse2(void *__restrict dst, const void *__restrict src, size_t count);
void *memcpy_sse3(void *__restrict dst, const void *__restrict src, size_t count);
void *memcpy_ssse3(void *__restrict dst, const void *__restrict src, size_t count);
void *memcpy_sse4(void *__restrict dst, const void *__restrict src, size_t count);
void *memcpy_sse41(void *__restrict dst, const void *__restrict src, size_t count);
void *memcpy_sse42(void *__restrict dst, const void *__restrict src, size_t count);
void *memcpy_avx(void *__restrict dst, const void *__restrict src, size_t count);

int main()
{
	clock_t start, end;
	unsigned long millis;
	
	char *buffer = malloc(10000);
	char *dest = malloc(10000);
	
	start = clock();
	for (int i = 0; i < 1000000; i++)
		memcpy_noopt(dest, buffer, 10000);
	end = clock();
	
	millis = (end - start) * 1000 / CLOCKS_PER_SEC;
	printf("No optimization: %ld\n", millis);

	start = clock();
	for (int i = 0; i < 1000000; i++)
		memcpy_mmx(dest, buffer, 10000);
	end = clock();
	
	millis = (end - start) * 1000 / CLOCKS_PER_SEC;
	printf("MMX: %ld\n", millis);

	start = clock();
	for (int i = 0; i < 1000000; i++)
		memcpy_sse(dest, buffer, 10000);
	end = clock();
	
	millis = (end - start) * 1000 / CLOCKS_PER_SEC;
	printf("SSE: %ld\n", millis);
	
	start = clock();
	for (int i = 0; i < 1000000; i++)
		memcpy_sse2(dest, buffer, 10000);
	end = clock();
	
	millis = (end - start) * 1000 / CLOCKS_PER_SEC;
	printf("SSE2: %ld\n", millis);
	
	start = clock();
	for (int i = 0; i < 1000000; i++)
		memcpy_sse3(dest, buffer, 10000);
	end = clock();
	
	millis = (end - start) * 1000 / CLOCKS_PER_SEC;
	printf("SSE3: %ld\n", millis);
	
	start = clock();
	for (int i = 0; i < 1000000; i++)
		memcpy_ssse3(dest, buffer, 10000);
	end = clock();
	
	millis = (end - start) * 1000 / CLOCKS_PER_SEC;
	printf("SSSE3: %ld\n", millis);
	
	start = clock();
	for (int i = 0; i < 1000000; i++)
		memcpy_sse4(dest, buffer, 10000);
	end = clock();
	
	millis = (end - start) * 1000 / CLOCKS_PER_SEC;
	printf("SSE4: %ld\n", millis);
	
	start = clock();
	for (int i = 0; i < 1000000; i++)
		memcpy_sse41(dest, buffer, 10000);
	end = clock();
	
	millis = (end - start) * 1000 / CLOCKS_PER_SEC;
	printf("SSE4.1: %ld\n", millis);
	
	start = clock();
	for (int i = 0; i < 1000000; i++)
		memcpy_sse42(dest, buffer, 10000);
	end = clock();
	
	millis = (end - start) * 1000 / CLOCKS_PER_SEC;
	printf("SSE4.2: %ld\n", millis);
	
	start = clock();
	for (int i = 0; i < 1000000; i++)
		memcpy_avx(dest, buffer, 10000);
	end = clock();
	
	millis = (end - start) * 1000 / CLOCKS_PER_SEC;
	printf("AVX: %ld\n", millis);

	free(dest);
	free(buffer);

	return 0;
}

